"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const camelCase = require("camel-case");
const constantCase = require("constant-case");
const fs = require("fs");
const merge = require("lodash.merge");
const path = require("path");
const TypeConf_1 = require("./TypeConf");
const util_1 = require("./util");
function readFile(file, parser) {
    try {
        const result = parser(fs.readFileSync(file, 'utf8'));
        if (typeof result === 'object' && result !== null)
            return result;
        console.error(`Invalid configuration file: ${file}`);
    }
    catch (e) {
        console.error(`Unable to read ${file}`);
    }
    return {};
}
function readConfigFile(filePath) {
    const extname = path.extname(filePath);
    switch (extname) {
        case '.yml':
        case '.yaml':
            return readFile(filePath, require('js-yaml').safeLoad);
        case '.json':
            return readFile(filePath, JSON.parse);
        default:
            console.error(`Unsupported file: ${filePath}\n`);
            return {};
    }
}
class TypeConfNode extends TypeConf_1.default {
    withArgv() {
        const argv = require('minimist')(process.argv.slice(2));
        const store = util_1.createStore(argv, camelCase);
        this.addStore(store, '__argv__');
        return this;
    }
    withEnv(prefix = '', separator = '__') {
        const prefixValue = constantCase(prefix);
        const getEnvName = (name) => {
            const keyValue = constantCase(name);
            return prefixValue ? `${prefixValue}_${keyValue}` : keyValue;
        };
        const getPartialKeys = (envName) => {
            return Object.keys(process.env).filter(key => key.startsWith(envName) && key.includes(separator));
        };
        const assignNestedProperty = (object, nestedKeys, value) => {
            if (nestedKeys.length === 1) {
                const [key] = nestedKeys;
                object[key] = value;
            }
            else {
                const [key, ...rest] = nestedKeys;
                object[key] = merge({}, object[key]);
                assignNestedProperty(object[key], rest, value);
            }
        };
        const getNestedKeys = (key) => key
            .split(separator)
            .slice(1)
            .map(s => camelCase(s));
        const getPartial = (envName) => {
            const partial = {};
            for (const key of getPartialKeys(envName)) {
                assignNestedProperty(partial, getNestedKeys(key), process.env[key]);
            }
            return partial;
        };
        const store = {
            get: (name, next) => {
                const envName = getEnvName(name);
                const result = process.env[envName];
                if (result !== undefined)
                    return result;
                const partial = getPartial(envName);
                if (Object.keys(partial).length > 0)
                    return merge({}, next(name), partial);
                else
                    return next(name);
            },
            has: (name, next) => {
                const envName = getEnvName(name);
                const result = envName in process.env || getPartialKeys(envName).length > 0;
                return result || next(name, true);
            }
        };
        const storeName = prefixValue ? `ENV_${prefixValue}` : 'ENV';
        this.addStore(store, storeName);
        return this;
    }
    withFile(file) {
        if (!file) {
            return this;
        }
        const filePath = path.resolve(process.cwd(), file);
        const storage = readConfigFile(filePath);
        const store = util_1.createStore(storage);
        super.addStore(store, filePath);
        return this;
    }
}
exports.default = TypeConfNode;
//# sourceMappingURL=TypeConfNode.js.map